// Versions:
// 25.11.2021 added setBitsSample(16) before playing 8-bit data 
//            unter Linux/Alsa (as we play always internal 16-bit data) 

// --------------------------------------------------------------
#include "waveclass10.h"

extern int verbose;

// Tests: read a WAVE file, store the data(samples) in memory
//        normalize and denormalize the data (samples)
//        play the in-memory data
void test1(const char *pFn,
           const char *pAlsaDev)
{
  WAVE *pWave = NULL;
  #ifndef WIN32
  ALSA *pAlsa = NULL;
  #endif
  long lRet = 0L;
  int  iRet = 0;
  //
  // allocate a WAVE instance and read the given WAVE file
  if (::verbose)
    ::printf("test1(): starting test1\n");
  pWave = new WAVE();
  pWave->setVerbose(::verbose);
  ::printf("test1(): reading file=%s\n", pFn);
  lRet = pWave->read(pFn);
  if (::verbose)
  {
    ::printf("test1(): result WAVE::read()=%ld\n", lRet);
    ::fflush(stdout);
  }
  if (::verbose)
  {
    ::printf("test1(): start playing...\n");
    ::fflush(stdout);
  }
  ::fflush(stdout);
  // play the WAVE file (which is now in memory)
  #ifndef WIN32
  // Linux here
  if (::verbose)
  {
    ::printf("test1() ALSA params: samplesSec=%lu channels=%u\n",
             pWave->getSamplesSec(), pWave->getNoChannels());
    ::fflush(stdout);
  }
  pAlsa = new ALSA(pAlsaDev, 
                   pWave->getSamplesSec(),
                   pWave->getNoChannels());
  //
  // as we play internal sample data (signed shorts):
  pWave->setBitsSample(16);  // TODO: should go into a constructor
  iRet = pAlsa->open(pWave, 0);   // must be!
  if (::verbose)
    ::printf("test1(): result ALSA::open()=%d\n", iRet);
  //
  pWave->play(pAlsa, 0L, 2000L);
  #else
  //
  // 32-bit Windows here
  pWave->play(0, 0L, 2000L);
  #endif
  if (::verbose)
    ::printf("test1(): end playing...\n");
  //
  zurueck:
  if (pWave != NULL)
    delete pWave;
  #ifndef WIN32
  if (pAlsa != NULL)
    delete pAlsa;
  #endif
  if (::verbose)
    ::printf("test1(): finishing test1\n");
  ::fflush(stdout);
}   // end test1()

// ----------------------------------------------------------------------
// Tests:
//  - read WAVE files
//  - normalize and denormalize samples
//  - write WAVE file in 16-bit and 8-bit format
void test2(const char *pFn,
           const char *pAlsaDev)
{
  long lRet = 0L;
  int  iRet = 0;
  WAVE *pWave = NULL;
  #ifndef WIN32
  ALSA *pAlsa = NULL;
  #endif
  //
  pWave = new WAVE();
  pWave->setVerbose(::verbose);
  ::printf("\ntest2(): reading file=%s\n", pFn);
  lRet = pWave->read(pFn);
  if (::verbose)
  {
    ::printf("test2(): result WAVE::read()=%ld\n", lRet);
    ::fflush(stdout);
  }
  if (::verbose)
    pWave->outputData("\ntest2() Before normalization:", 512L);
  lRet = pWave->normalizeData();
  if (::verbose)
    pWave->outputData("\ntest2() After normalization:", 512L);
  // Absturz hier nur bei stereo???
  if (::verbose)
    pWave->outputData("\ntest2() Before deNormalization:", 512L);
  pWave->deNormalizeData();
  if (::verbose)
    pWave->outputData("\ntest2() After deNormalization:", 512L);
  //
  ::printf("test2(): start playing...\n");
  ::fflush(stdout);
  // play the (modified) samples
  #ifndef WIN32
  // Linux here
  if (::verbose)
  {
    ::printf("test2() ALSA params: samplesSec=%lu channels=%u\n",
             pWave->getSamplesSec(), pWave->getNoChannels());
    ::fflush(stdout);
  }
  pAlsa = new ALSA(pAlsaDev, 
                   pWave->getSamplesSec(),
                   pWave->getNoChannels());
  //
  // as we play internal sample data (signed shorts):
  pWave->setBitsSample(16);  // TODO: should go into a constructor
  iRet = pAlsa->open(pWave, 0);   // must be!
  if (::verbose)
    ::printf("test2(): result ALSA::open()=%d\n", iRet);
  //
  pWave->play(pAlsa, 0L, 2000L);
  #else
  // 32-bit Windows here
  pWave->setBitsSample(16);  
  pWave->play(0, 0L, 2000L);
  #endif
  ::printf("\ntest2(): writing new WAVE file test2_x.wav...\n");
  #ifndef WIN32
  // Linux
  pWave->setVerbose(::verbose);
  lRet = pWave->write("./test2_16.wav");
  lRet = pWave->write("./test2_8.wav", 8);
  #else
  // Windows
  lRet = pWave->write(".\\test2_16.wav");
  lRet = pWave->write(".\\test2_8.wav", 8);
  #endif
  ::printf("test2() written %lu bytes to file test2_x.wav\n", lRet);
  ::fflush(stdout);
  //
  delete pWave;
  #ifndef WIN32
  if (pAlsa != NULL)
    delete pAlsa;
  #endif
}   // end test2()
 
